@@ -846,10 +846,11 @@ define_or_generate(Pred) :-
     '$get_predicate_attribute'(Pred, quasi_quotation_syntax, 1).
 '$predicate_property'(defined, Pred) :-
     '$get_predicate_attribute'(Pred, defined, 1).
-'$predicate_property'(tabled(Mode), Pred) :-
+'$predicate_property'(tabled, Pred) :-
+    '$get_predicate_attribute'(Pred, tabled, 1).
+'$predicate_property'(tabled(Flag), Pred) :-
     '$get_predicate_attribute'(Pred, tabled, 1),
-    '$tbl_implementation'(Pred, M:Head),
-    M:'$tabled'(Head, Mode).
+    table_flag(Flag, Pred).
 '$predicate_property'(incremental, Pred) :-
     '$get_predicate_attribute'(Pred, incremental, 1).
 '$predicate_property'(abstract(0), Pred) :-
@@ -860,6 +861,18 @@ system_undefined(user:prolog_exception_hook/4).
 system_undefined(system:'$c_call_prolog'/0).
 system_undefined(system:window_title/2).
 
+table_flag(variant, Pred) :-
+    '$tbl_implementation'(Pred, M:Head),
+    M:'$tabled'(Head, variant).
+table_flag(subsumptive, Pred) :-
+    '$tbl_implementation'(Pred, M:Head),
+    M:'$tabled'(Head, subsumptive).
+table_flag(shared, Pred) :-
+    '$get_predicate_attribute'(Pred, tshared, 1).
+table_flag(incremental, Pred) :-
+    '$get_predicate_attribute'(Pred, incremental, 1).
+
+
 %!  visible_predicate(:Head) is nondet.
 %
 %   True when Head can be called without raising an existence error.
@@ -1146,6 +1159,8 @@ current_trie(Trie) :-
 %     Number of bytes needed to store the trie.
 %     - hashed(Count)
 %     Number of hashed nodes.
+%     - compiled_size(Bytes)
+%     Size of the compiled representation (if the trie is compiled)
 
 trie_property(Trie, Property) :-
     current_trie(Trie),
@@ -1156,6 +1171,7 @@ trie_property(node_count(_)).
 trie_property(value_count(_)).
 trie_property(size(_)).
 trie_property(hashed(_)).
+trie_property(compiled_size(_)).
                                                 % below only when -DO_TRIE_STATS
 trie_property(lookup_count(_)).                 % is enabled in pl-trie.h
 trie_property(gen_call_count(_)).
@@ -1467,7 +1483,7 @@ set_prolog_gc_thread(Status) :-
 '$wrap_predicate'(M:Head, WName, Closure, call(Wrapped), Body) :-
     callable_name_arguments(Head, PName, Args),
     distinct_vars(Args, Head, Arity),
-    atomic_list_concat(['__wrap$', PName], WrapName),
+    atomic_list_concat(['$wrap$', PName], WrapName),
     volatile(M:WrapName/Arity),
     WHead =.. [WrapName|Args],
     '$c_wrap_predicate'(M:Head, WName, Closure, Wrapped, M:(WHead :- Body)).
