@@ -120,6 +120,14 @@ delay_goals(List, Goal) :-
 user_goal(Goal, UGoal) :-
     unqualify_goal(Goal, user, UGoal).
 
+:- multifile
+    prolog:portray/1.
+
+user:portray(ATrie) :-
+    '$is_answer_trie'(ATrie),
+    trie_goal(ATrie, Goal, _Skeleton),
+    format('~q for ~p', [ATrie, Goal]).
+
 :- endif.
 
 %!  table(:PredicateIndicators)
@@ -228,7 +236,7 @@ untable(Name//Arity, M) :-
 untable(Name/Arity, M) :-
     !,
     functor(Head, Name, Arity),
-    (   predicate_property(M:Head, tabled(_))
+    (   '$get_predicate_attribute'(M:Head, tabled, 1)
     ->  abolish_table_subgoals(M:Head),
         dynamic(M:'$tabled'/2),
         dynamic(M:'$table_mode'/3),
@@ -246,6 +254,13 @@ untable(Head, M) :-
 untable(TableSpec, _) :-
     '$type_error'(table_desclaration, TableSpec).
 
+untable_reconsult(PI) :-
+    print_message(informational, untable(PI)),
+    untable(PI).
+
+:- initialization
+   prolog_listen(untable, untable_reconsult).
+
 
 %!  start_tabling(:Wrapper, :Implementation)
 %
@@ -295,8 +310,7 @@ start_tabling(Closure, Wrapper, Worker) :-
               deadlock,
               restart_tabling(Closure, Wrapper, Worker))
     ;   Status == invalid
-    ->  reeval(Trie),                           % needs clause
-        trie_gen_compiled(Trie, Skeleton)
+    ->  reeval(Trie, Wrapper, Skeleton)
     ;   % = run_follower, but never fresh and Status is a worklist
         shift(call_info(Skeleton, Status))
     ).
@@ -848,7 +862,12 @@ current_table(M:Variant, Trie) :-
     system:term_expansion/2.
 
 wrappers(Spec, M) -->
-    wrappers(Spec, M, #{}).
+    { tabling_defaults([ table_incremental-incremental,
+                         table_shared-tshared
+                       ],
+                       #{}, Defaults)
+    },
+    wrappers(Spec, M, Defaults).
 
 wrappers(Var, _, _) -->
     { var(Var),
@@ -926,6 +945,15 @@ qlist([], _)    --> [].
 qlist([H|T], M) --> [M:H], qlist(T, M).
 
 
+tabling_defaults([], Dict, Dict).
+tabling_defaults([Flag-Opt|T], Dict0, Dict) :-
+    (   current_prolog_flag(Flag, true)
+    ->  Dict1 = Dict0.put(Opt,true)
+    ;   Dict1 = Dict0
+    ),
+    tabling_defaults(T, Dict1, Dict).
+
+
 %!  table_options(+Options, +OptDictIn, -OptDictOut)
 %
 %   Handler the ... as _options_ ... construct.
@@ -946,6 +974,9 @@ table_options(variant, Opts0, Opts1) :-
 table_options(incremental, Opts0, Opts1) :-
     !,
     put_dict(incremental, Opts0, true, Opts1).
+table_options(opaque, Opts0, Opts1) :-
+    !,
+    put_dict(incremental, Opts0, false, Opts1).
 table_options(dynamic, Opts0, Opts1) :-
     !,
     put_dict(dynamic, Opts0, true, Opts1).
@@ -1195,11 +1226,12 @@ dyn_update(Abstract, _, _) :-
     dyn_changed_pattern(Abstract).
 
 dyn_changed_pattern(Term) :-
-    '$tbl_variant_table'(VTable),
-    !,
-    forall(trie_gen(VTable, Term, ATrie),
+    forall(dyn_affected(Term, ATrie),
            '$idg_changed'(ATrie)).
-dyn_changed_pattern(_).
+
+dyn_affected(Term, ATrie) :-
+    '$tbl_variant_table'(VTable),
+    trie_gen(VTable, Term, ATrie).
 
 %!  '$unwrap_incremental'(:Head) is det.
 %
@@ -1223,31 +1255,65 @@ dyn_changed_pattern(_).
     ;   true
     ).
 
-%!  reeval(+ATrie)
+%!  reeval(+ATrie, :Goal, ?Return) is nondet.
 %
 %   Called  if  the   table   ATrie    is   out-of-date   (has  non-zero
-%   _falsecount_). This finds all dependency paths to dynamic predicates
-%   and then evaluates the nodes in   a breath-first fashion starting at
-%   the level just above the  dynamic   predicates  and  moving upwards.
-%   Bottom up evaluation is used to   profit  from upward propagation of
-%   not-modified events that may cause the evaluation to stop early.
+%   _falsecount_). The answers of this predicate are the answers to Goal
+%   after re-evaluating the answer trie.
+%
+%   This finds all dependency  paths  to   dynamic  predicates  and then
+%   evaluates the nodes in a breath-first  fashion starting at the level
+%   just above the dynamic predicates  and   moving  upwards.  Bottom up
+%   evaluation is used to profit from upward propagation of not-modified
+%   events that may cause the evaluation to stop early.
 %
 %   Note that false paths either end  in   a  dynamic node or a complete
 %   node. The latter happens if we have and  IDG   "D  -> P -> Q" and we
 %   first re-evaluate P for some reason.  Now   Q  can  still be invalid
 %   after P has been re-evaluated.
+%
+%   @arg ATrie is the answer trie.  When shared tabling, we own this
+%   trie.
+%   @arg Goal is tabled goal (variant).  If we run into a deadlock we
+%   need to call this.
+%   @arg Return is the return skeleton. We must run
+%   trie_gen_compiled(ATrie, Return) to enumerate the answers
 
-reeval(ATrie) :-
+reeval(ATrie, Goal, Return) :-
+    catch(try_reeval(ATrie, Goal, Return), deadlock,
+          retry_reeval(ATrie, Goal)).
+
+retry_reeval(ATrie, Goal) :-
+    '$tbl_reeval_abandon'(ATrie),
+    tdebug(deadlock, 'Deadlock re-evaluating ~p; retrying', [ATrie]),
+    sleep(0.000001),
+    call(Goal).
+
+try_reeval(ATrie, Goal, Return) :-
+    nb_current('$tbl_reeval', true),
+    !,
+    tdebug(reeval, 'Nested re-evaluation for ~p', [ATrie]),
+    '$tbl_reeval_prepare'(ATrie, _Variant, Clause),
+    (   nonvar(Clause)
+    ->  trie_gen_compiled(Clause, Return)
+    ;   call(Goal)
+    ).
+try_reeval(ATrie, Goal, Return) :-
     tdebug(reeval, 'Planning reeval for ~p', [ATrie]),
     findall(Path, false_path(ATrie, Path), Paths0),
     sort(0, @>, Paths0, Paths),
     split_paths(Paths, Dynamic, Complete),
     tdebug(forall('$member'(Path, Dynamic),
-                  tdebug(reeval, 'Re-eval dynamic path: ~p', [Path]))),
+                  tdebug(reeval, '  Re-eval dynamic path: ~p', [Path]))),
     tdebug(forall('$member'(Path, Complete),
-                  tdebug(reeval, 'Re-eval complete path: ~p', [Path]))),
+                  tdebug(reeval, '  Re-eval complete path: ~p', [Path]))),
     reeval_paths(Dynamic, ATrie),
-    reeval_paths(Complete, ATrie).
+    reeval_paths(Complete, ATrie),
+    '$tbl_reeval_prepare'(ATrie, _Variant, Clause),
+    (   nonvar(Clause)
+    ->  trie_gen_compiled(Clause, Return)
+    ;   call(Goal)
+    ).
 
 split_paths([], [], []).
 split_paths([[Rank-_Len|Path]|T], [Path|DT], CT) :-
@@ -1282,6 +1348,15 @@ reeval_heads([[H|T]|B], ATrie, [T|BT]) :-
     reeval_node(H),
     reeval_heads(B, ATrie, BT).
 
+%!  false_path(+Atrie, -Path) is nondet.
+%
+%   True when Path is a list of   invalid  tries (bottom up, ending with
+%   ATrie). The last element of the list is a term `Rank-Length` that is
+%   used for sorting the paths.
+%
+%   If we find a table along the  way   that  is being worked on by some
+%   other thread we wait for it.
+
 false_path(ATrie, BottomUp) :-
     false_path(ATrie, Path, []),
     '$reverse'(Path, BottomUp).
@@ -1289,7 +1364,8 @@ false_path(ATrie, BottomUp) :-
 false_path(ATrie, [ATrie|T], Seen) :-
     \+ memberchk(ATrie, Seen),
     '$idg_edge'(ATrie, dependent, Dep),
-    '$tbl_table_status'(Dep, Status, _, _),
+    '$tbl_reeval_wait'(Dep, Status),
+    tdebug(reeval, '    ~p has dependent ~p (~w)', [ATrie, Dep, Status]),
     (   Status == invalid
     ->  false_path(Dep, T, [ATrie|Seen])
     ;   status_rank(Status, Rank),
@@ -1299,8 +1375,13 @@ false_path(ATrie, [ATrie|T], Seen) :-
 
 status_rank(dynamic,  2) :- !.
 status_rank(complete, 1) :- !.
-status_rank(Status,   0) :-
-    format(user_error, 'Re-eval from status ~p~n', [Status]).
+status_rank(Status,   Rank) :-
+    var(Rank),
+    !,
+    format(user_error, 'Re-eval from status ~p~n', [Status]),
+    Rank = 0.
+status_rank(Rank,   Rank) :-
+    format(user_error, 'Re-eval from rank ~p~n', [Rank]).
 
 is_invalid(ATrie) :-
     '$idg_falsecount'(ATrie, FalseCount),
@@ -1308,23 +1389,27 @@ is_invalid(ATrie) :-
 
 %!  reeval_node(+ATrie)
 %
-%   Re-evaluate the invalid answer  trie  ATrie.   This  creates  a  sub
-%   tabling environment and solves the variant associated with ATrie.
+%   Re-evaluate the invalid answer trie ATrie.  Initially this created a
+%   nested tabling environment, but this is dropped:
+%
+%     - It is possible for the re-evaluating variant to call into outer
+%       non/not-yet incremental tables, requiring a merge with this
+%       outer SCC.  This doesn't work well with a sub-environment.
+%     - We do not need one.  If this environment is not merged into the
+%       outer one it will complete before we continue.
 
 reeval_node(ATrie) :-
-    is_invalid(ATrie),
+    '$tbl_reeval_prepare'(ATrie, Variant, Clause),
+    var(Clause),
     !,
-    tdebug(trie_goal(ATrie, Goal, _)),
-    tdebug(reeval, 'Re-evaluating ~p', [Goal]),
-    '$tbl_reeval_prepare'(ATrie),
-    '$tbl_table_status'(ATrie, _, Variant, _),
-    (   '$idg_reset_current',                   % move to '$tbl_scc_save'/1?
+    tdebug(reeval, 'Re-evaluating ~p', [Variant]),
+    (   '$idg_reset_current',
         setup_call_cleanup(
-            '$tbl_scc_save'(State),
-            call(Variant),
-            '$tbl_scc_restore'(State)),
+            nb_setval('$tbl_reeval', true),
+            ignore(Variant),                    % assumes local scheduling
+            nb_delete('$tbl_reeval')),
         fail
-    ;   true
+    ;   tdebug(reeval, 'Re-evaluated ~p', [Variant])
     ).
 reeval_node(_).
 
@@ -1480,17 +1565,9 @@ eval_dl_in_residual(G) :-
 
 more_general_table(G, Trie) :-
     term_variables(G, Vars),
-    length(Vars, Len),
     '$tbl_variant_table'(VariantTrie),
     trie_gen(VariantTrie, G, Trie),
-    all_vars(Vars),
-    sort(Vars, V2),
-    length(V2, Len).
-
-all_vars([]).
-all_vars([H|T]) :-
-    var(H),
-    all_vars(T).
+    is_most_general_term(Vars).
 
 :- table eval_subgoal_in_residual/2.
 
