@@ -3,7 +3,8 @@
     Author:        Jan Wielemaker and Richard O'Keefe
     E-mail:        J.Wielemaker@cs.vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2014-2017, VU University Amsterdam
+    Copyright (c)  2014-2018, VU University Amsterdam
+                              CWI, Amsterdam
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -34,8 +35,11 @@
 
 :- module(check_installation,
           [ check_installation/0,
-            check_installation/1                % -Issues
+            check_installation/1,               % -Issues
+            test_installation/0,
+            test_installation/1                 % +Options
           ]).
+:- use_module(option).
 
 /** <module> Check installation issues and features
 
@@ -93,6 +97,7 @@ component(library(pcre), _{features:pcre_features}).
 component(library(pdt_console), _{}).
 component(library(porter_stem), _{}).
 component(library(process), _{}).
+component(library(protobufs), _{}).
 component(library(editline), _{os:unix}).
 component(library(readline), _{os:unix}).
 component(library(readutil), _{}).
@@ -105,6 +110,7 @@ component(library(sha), _{}).
 component(library(snowball), _{}).
 component(library(socket), _{}).
 component(library(ssl), _{}).
+component(library(crypto), _{}).
 component(library(syslog), _{os:unix}).
 component(library(table), _{}).
 component(library(time), _{}).
@@ -132,6 +138,8 @@ issue_base('http://www.swi-prolog.org/build/issues/').
 %        are present (e.g., unbounded arithmetic support)
 %     2. Test that all standard libraries that depend on foreign
 %        code are present.
+%     3. provides a test_installation predicate to run the tests
+%        at runtime if the system was built with -DINSTALL_TESTS
 %
 %   If issues are found it prints a   diagnostic message with a link
 %   to a wiki page with additional information about the issue.
@@ -146,7 +154,7 @@ check_installation :-
         print_message(warning, installation(imperfect(Count)))
     ).
 
-%!  check_insmtallation(-Issues:list(pair)) is det.
+%!  check_installation(-Issues:list(pair)) is det.
 %
 %   As check_installation/0, but additionally  returns   a  list  of
 %   Component-Problem pairs. Problem is  one of `optional_not_found`
@@ -385,8 +393,13 @@ jquery_file :-
 %   Validate that Prolog is installed in $PATH
 
 check_on_path :-
-    current_prolog_flag(executable, EXE),
+    current_prolog_flag(executable, EXEFlag),
+    prolog_to_os_filename(EXE, EXEFlag),
     file_base_name(EXE, Prog),
+    absolute_file_name(EXE, AbsExe,
+                       [ access(execute)
+                       ]),
+    prolog_to_os_filename(AbsExe, OsExe),
     (   absolute_file_name(path(Prog), OnPath,
                            [ access(execute),
                              file_errors(fail)
@@ -399,13 +412,65 @@ check_on_path :-
                                  solutions(all)
                                ]),
             same_file(EXE, OnPathAny)
-        ->  print_message(warning, installation(not_first_on_path(EXE, OnPath)))
-        ;   print_message(warning, installation(not_same_on_path(EXE, OnPath)))
+        ->  print_message(warning, installation(not_first_on_path(OsExe, OnPath)))
+        ;   print_message(warning, installation(not_same_on_path(OsExe, OnPath)))
         )
-    ;   print_message(warning, installation(not_on_path(EXE, Prog)))
+    ;   print_message(warning, installation(not_on_path(OsExe, Prog)))
     ).
 
 
+		 /*******************************
+		 *           RUN TESTS		*
+		 *******************************/
+
+%!  test_installation is semidet.
+%!  test_installation(+Options) is semidet.
+%
+%   Run regression tests in the installed system. Requires the system to
+%   be built using
+%
+%	cmake -DINSTALL_TESTS=ON
+%
+%   Options processed:
+%
+%     - packages(+Boolean)
+%       When `false`, do not test the packages
+%     - package(+Package)
+%       Only test package package.
+
+test_installation :-
+    test_installation([]).
+
+test_installation(Options) :-
+    absolute_file_name(swi(test/test),
+                       TestFile,
+                       [ access(read),
+                         file_errors(fail),
+                         file_type(prolog)
+                       ]),
+    !,
+    test_installation_run(TestFile, Options).
+test_installation(_Options) :-
+    print_message(warning, installation(testing(no_installed_tests))).
+
+test_installation_run(TestFile, Options) :-
+    (   option(package(_), Options)
+    ->  merge_options(Options,
+                      [ core(false),
+                        subdirs(false)
+                      ], TestOptions)
+    ;   merge_options(Options,
+                      [ packages(true)
+                      ], TestOptions)
+    ),
+    load_files(user:TestFile),
+    current_prolog_flag(verbose, Old),
+    setup_call_cleanup(
+        set_prolog_flag(verbose, silent),
+        user:test([], TestOptions),
+        set_prolog_flag(verbose, Old)).
+
+
                  /*******************************
                  *            MESSAGES          *
                  *******************************/
@@ -505,6 +570,9 @@ message(jquery(found(Path))) -->
     [ '  jQuery from ~w'-[Path] ].
 message(jquery(not_found(File))) -->
     [ '  Cannot find jQuery (~w)'-[File] ].
+message(testing(no_installed_tests)) -->
+    [ '  Runtime testing is not enabled.', nl],
+    [ '  Please recompile the system with INSTALL_TESTS enabled.' ].
 
 
 public_executable(EXE, PublicProg) :-
