@@ -229,7 +229,7 @@ prolog_walk_code(Iteration, Options) :-
 is_meta(on_trace).
 
 
-%!  walk_clauses(Clauses, +OTerm) is det.
+%!  walk_clauses(+Clauses, +OTerm) is det.
 %
 %   Walk the given clauses.
 
@@ -515,6 +515,20 @@ walk_called(Meta, M, term_position(_,E,_,_,ArgPosList), OTerm) :-
     walk_option_clause(OTerm, ClauseRef),
     register_possible_meta_clause(ClauseRef),
     walk_meta_call(1, Head, Meta, M, ArgPosList, E-E, OTerm).
+walk_called(Closure, _, _, _) :-
+    blob(Closure, closure),
+    !,
+    '$closure_predicate'(Closure, Module:Name/Arity),
+    functor(Head, Name, Arity),
+    '$get_predicate_attribute'(Module:Head, defined, 1).
+walk_called(ClosureCall, _, _, _) :-
+    compound(ClosureCall),
+    functor(ClosureCall, Closure, _),
+    blob(Closure, closure),
+    !,
+    '$closure_predicate'(Closure, Module:Name/Arity),
+    functor(Head, Name, Arity),
+    '$get_predicate_attribute'(Module:Head, defined, 1).
 walk_called(Goal, Module, _, _) :-
     nonvar(Module),
     '$get_predicate_attribute'(Module:Goal, defined, 1),
@@ -903,6 +917,17 @@ extend(Goal, N, GoalEx, TermPosIn, TermPosOut, _) :-
     extend_term_pos(TermPosIn, N, TermPosOut),
     append(List, Extra, ListEx),
     GoalEx =.. ListEx.
+extend(Closure, N, M:GoalEx, TermPosIn, TermPosOut, OTerm) :-
+    blob(Closure, closure),             % call(Closure, A1, ...)
+    !,
+    '$closure_predicate'(Closure, M:Name/Arity),
+    length(Extra, N),
+    extend_term_pos(TermPosIn, N, TermPosOut),
+    GoalEx =.. [Name|Extra],
+    (   N =:= Arity
+    ->  true
+    ;   print_reference(Closure, TermPosIn, closure_arity_mismatch, OTerm)
+    ).
 extend(Goal, _, _, TermPos, _, OTerm) :-
     print_reference(Goal, TermPos, not_callable, OTerm).
 
