@@ -182,7 +182,7 @@ static findall_bag *
 current_bag(ARG1_LD)
 { findall_bag *bag = LD->bags.bags;
 
-  while(bag->suspended)
+  while(bag && bag->suspended)
   { assert(bag->parent);
     bag = bag->parent;
   }
@@ -203,6 +203,16 @@ add_findall_bag(term_t term, term_t count ARG_LD)
 				   PL_WRT_QUOTED);
 		   });
 
+  if ( !bag )
+  { static atom_t cbag;
+
+    if ( !cbag )
+      cbag = PL_new_atom("findall-bag");
+
+    return PL_error(NULL, 0, "continuation in findall/3 generator?",
+		    ERR_PERMISSION, ATOM_append, cbag, term);
+  }
+
   if ( !(r = compileTermToHeap__LD(term, alloc_record, bag, R_NOLOCK PASS_LD)) )
     return PL_no_memory();
   if ( !pushRecordSegStack(&bag->answers, r) )
@@ -210,7 +220,7 @@ add_findall_bag(term_t term, term_t count ARG_LD)
   bag->gsize += r->gsize;
   bag->solutions++;
 
-  if ( bag->gsize + bag->solutions*3 > limitStack(global)/sizeof(word) )
+  if ( bag->gsize + bag->solutions*3 > globalStackLimit()/sizeof(word) )
     return outOfStack(&LD->stacks.global, STACK_OVERFLOW_RAISE);
 
   if ( count )
@@ -296,7 +306,9 @@ PRED_IMPL("$suspend_findall_bag", 0, suspend_findall_bag, PL_FA_NONDETERMINISTIC
   switch( CTX_CNTRL )
   { case FRG_FIRST_CALL:
       bag = current_bag(PASS_LD1);
+      simpleMutexLock(&LD->bags.mutex);
       clear_mem_pool(&bag->records);
+      simpleMutexUnlock(&LD->bags.mutex);
       bag->suspended_solutions += bag->solutions;
       bag->solutions = 0;
       bag->gsize = 0;
