@@ -1038,6 +1038,25 @@ then ->next because ->next might be used by some other thread traversing
 the clause chain.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
+void
+acquire_clause(Clause cl)
+{ ATOMIC_INC(&cl->references);
+}
+
+void
+release_clause(Clause cl)
+{ if ( ATOMIC_DEC(&cl->references) == 0 )
+  { size_t size = sizeofClause(cl->code_size) + SIZEOF_CREF_CLAUSE;
+
+    ATOMIC_SUB(&GD->clauses.erased_size, size);
+    ATOMIC_DEC(&GD->clauses.erased);
+
+    reclaimRetracted(cl);
+    freeClause(cl);
+  }
+}
+
+
 ClauseRef
 newClauseRef(Clause clause, word key)
 { ClauseRef cref = allocHeapOrHalt(SIZEOF_CREF_CLAUSE);
@@ -1050,7 +1069,7 @@ newClauseRef(Clause clause, word key)
   cref->next         = NULL;
   cref->d.key        = key;
   cref->value.clause = clause;
-  ATOMIC_INC(&clause->references);
+  acquire_clause(clause);
 
   return cref;
 }
@@ -1064,15 +1083,7 @@ freeClauseRef(ClauseRef cref)
 	Sdprintf("/**/ d(%p, %p, %d).\n",
 		 cref, cl, (int)cl->references));
 
-  if ( ATOMIC_DEC(&cl->references) == 0 )
-  { size_t size = sizeofClause(cl->code_size) + SIZEOF_CREF_CLAUSE;
-
-    ATOMIC_SUB(&GD->clauses.erased_size, size);
-    ATOMIC_DEC(&GD->clauses.erased);
-
-    reclaimRetracted(cl);
-    freeClause(cl);
-  }
+  release_clause(cl);
 
   freeHeap(cref, SIZEOF_CREF_CLAUSE);
 }
@@ -3082,9 +3093,13 @@ get_bool_or_int_ex(term_t t, int *val ARG_LD)
 }
 
 
-word
-pl_set_predicate_attribute(term_t pred, term_t what, term_t value)
-{ GET_LD
+static
+PRED_IMPL("$set_predicate_attribute", 3, set_predicate_attribute,
+	  PL_FA_TRANSPARENT)
+{ PRED_LD
+  term_t pred  = A1;
+  term_t what  = A2;
+  term_t value = A3;
   Procedure proc;
   Definition def;
   atom_t key;
@@ -3105,7 +3120,7 @@ pl_set_predicate_attribute(term_t pred, term_t what, term_t value)
   }
   def = proc->definition;
 
-  if ( ReadingSource )
+  if ( ReadingSource && MODULE_parse == def->module )
   { SourceFile sf = lookupSourceFile(source_file_name, TRUE);
     return setAttrProcedureSource(sf, proc, att, val PASS_LD);
   } else
@@ -3114,12 +3129,12 @@ pl_set_predicate_attribute(term_t pred, term_t what, term_t value)
 }
 
 
-word
-pl_default_predicate(term_t d1, term_t d2)
+static
+PRED_IMPL("$default_predicate", 2, default_predicate, PL_FA_TRANSPARENT)
 { Procedure p1, p2;
 
-  if ( get_procedure(d1, &p1, 0, GP_FIND) &&
-       get_procedure(d2, &p2, 0, GP_FIND) )
+  if ( get_procedure(A1, &p1, 0, GP_FIND) &&
+       get_procedure(A2, &p2, 0, GP_FIND) )
   { if ( p1->definition == p2->definition || !isDefinedProcedure(p1) )
       succeed;
   }
@@ -3476,6 +3491,9 @@ pl_list_generations(term_t desc)
 		 *******************************/
 
 BeginPredDefs(proc)
+  PRED_DEF("$set_predicate_attribute", 3, set_predicate_attribute,
+	   PL_FA_TRANSPARENT)
+  PRED_DEF("$default_predicate", 2, default_predicate, PL_FA_TRANSPARENT)
   PRED_DEF("meta_predicate", 1, meta_predicate, PL_FA_TRANSPARENT)
   PRED_DEF("$get_clause_attribute", 3, get_clause_attribute, 0)
   PRED_DEF("retract", 1, retract,
