@@ -74,12 +74,6 @@ TODO
 
 #define NVARS_FAST 100
 
-#ifdef O_TRIE_STATS
-#define TRIE_STAT_INC(t, v) ATOMIC_INC(&t->stats.v)
-#else
-#define TRIE_STAT_INC(t, v) ((void)0)
-#endif
-
 /* Will eventually be shared in pl-wam.c */
 typedef enum
 { uread = 0,				/* Unification in read-mode */
@@ -220,8 +214,9 @@ trie_discard_clause(trie *trie)
 { atom_t dbref;
 
   if ( (dbref=trie->clause) )
-  { if ( COMPARE_AND_SWAP(&trie->clause, dbref, 0) )
-    { ClauseRef cref = clause_clref(dbref);
+  { if ( COMPARE_AND_SWAP(&trie->clause, dbref, 0) &&
+	 GD->cleaning == CLN_NORMAL )		/* otherwise reclaims clause from */
+    { ClauseRef cref = clause_clref(dbref);	/* two ends */
 
       if ( cref )
       { Clause cl = cref->value.clause;
@@ -244,6 +239,8 @@ trie_empty(trie *trie)
     clear_node(trie, &trie->root, FALSE);	/* TBD: verify not accessed */
     if ( it && COMPARE_AND_SWAP(&trie->indirects, it, NULL) )
       destroy_indirect_table(it);
+    trie->node_count = 1;
+    trie->value_count = 0;
   }
 }
 
@@ -1642,6 +1639,9 @@ add_choice(trie_gen_state *state, descent_state *dstate, trie_node *node ARG_LD)
 	     IS_TRIE_KEY_POP(children.key->key) )
 	{ word key   = children.key->key;
 
+	  if ( tagex(children.key->key) == TAG_VAR )
+	    dstate->prune = FALSE;
+
 	  ch = allocFromBuffer(&state->choicepoints, sizeof(*ch));
 	  ch->key    = key;
 	  ch->child  = children.key->child;
@@ -1982,6 +1982,16 @@ PRED_IMPL("$trie_property", 2, trie_property, 0)
 	// assert(stats.nodes  == trie->node_count);
 	// assert(stats.values == trie->value_count);
 	return PL_unify_int64(arg, stats.bytes);
+      } else if ( name == ATOM_compiled_size )
+      { atom_t dbref;
+	if ( (dbref = trie->clause) )
+	{ ClauseRef cref = clause_clref(dbref);
+	  if ( cref )
+	  { size_t sz = sizeofClause(cref->value.clause->code_size);
+	    return PL_unify_int64(arg, sz);
+	  }
+	}
+	return FALSE;
       } else if ( name == ATOM_hashed )
       { trie_stats stats;
 	stat_trie(trie, &stats);
